//
//  BFRecordScreenCameraManager.swift
//  BFRecordScreenKit
//
//  Created by 胡志强 on 2022/1/13.
//

/*
 
 时长是根据摄像头文件时长相加得来的，预览/合成时需要考虑录音的时长问题
 
 videosticker 已经存储好了，在合成时，需要更新下进入的时间就可以了，不需要再切割了
 
 
 
 */

import Foundation
import GPUImage
import BFCommonKit
import BFUIKit
import BFMediaKit
import CoreMedia
import UIKit


let vpath = recordVideosDirectory + "camera_writer.mov"
let videoSize = CGSize(width: cScreenWidth, height: cScreenWidth * 16 / 9.0)

class BFRecordScreenCameraManager : BFRecordScreenBaseManager{
    //
    var rendView = GPUImageView()
    var cameraProgressV: BFCameraProgressView?
    
    // 录制完成回调
    var hasInitCallBack : (()->Void)?
    var recordEndCallBack : recordEndCallBack?
    var recordProgressCallBack : ((CMTime) -> Void)?
    
    var startTime = Date()
    var videoModel = PQEditVisionTrackMaterialsModel()
    
    // 用于打开摄像头初始流程
    static var firstOpenCamera : Bool = true
    //当前录制的缩略图数量
    var currVideoThumImagesNum = 0
    //当前摄像的序号
    var currVideoIndex = 0
    
    fileprivate var timerr:Timer?
   
    var movieWrite : GPUImageMovieWriter?
    
    lazy var camera : GPUImageStillCamera? = {
        let camera = GPUImageStillCamera(sessionPreset: AVCaptureSession.Preset.hd1920x1080.rawValue, cameraPosition: AVCaptureDevice.Position.back)
        camera?.outputImageOrientation = UIInterfaceOrientation.portrait
//        camera?.addAudioInputsAndOutputs()
        camera?.delegate = self
        return camera
        
    }()
    
    
    override func resetEnv(){
        guard let camera = camera else {
            cShowHUB(superView: nil, msg: "摄像头开启失败！")
            return
        }
        cameraProgressV?.recordItem = recordItem
        cameraProgressV?.collectionV.reloadData()
        cameraProgressV?.isHidden = true
        currVideoIndex = 0

        camera.startCapture()
        initerlizeWriter()

        camera.addTarget(filter)
        filter.addTarget(rendView)
        
        if BFRecordScreenCameraManager.firstOpenCamera {
            startRecord()
            DispatchQueue.main.asyncAfter(deadline: .now() + 0.3, execute: { [weak self] in
                guard let wself = self else { return }

                wself.endRecord()
                BFRecordScreenCameraManager.firstOpenCamera = false
            })
        }
    }
    
    override func startRecord(){
        guard let movieWrite = movieWrite else {
            cShowHUB(superView: nil, msg: "录制启动失败")
            return
        }
        

        super.startRecord()

        currVideoThumImagesNum = 0
        startTime = Date()
        
        videoModel = PQEditVisionTrackMaterialsModel()
        videoModel.thumImgs = [UIImage]()
        videoModel.timelineIn = currentAssetProgress.seconds
        videoModel.timelineOut = currentAssetProgress.seconds
        currVideoIndex = recordItem?.videoStickers.count ?? 0
        
        
        // 为了第一时间能更新collection view数据
        if !BFRecordScreenCameraManager.firstOpenCamera{
            recordItem?.videoStickers.append(videoModel)
            progreddL?.isHidden = false
            cameraProgressV?.isHidden = false
        }
        movieWrite.startRecording()
        
        timerr = Timer.scheduledTimer(withTimeInterval: 0.01, repeats: true, block: {[weak self] _ in
            guard let wself = self else { return }
            if BFRecordScreenCameraManager.firstOpenCamera {
                return
            }
            //MARK: 进度计时器

            let currDur = CMTime(seconds: wself.videoModel.timelineIn + movieWrite.duration.seconds, preferredTimescale: 1000)
            wself.currentAssetProgress = currDur
            wself.videoModel.timelineOut = currDur.seconds
            wself.recordProgressCallBack?(currDur)
            wself.updateUI(progress: currDur)
            BFLog(1, message: "currTime: \(wself.currentAssetProgress.seconds)")
        })
        RunLoop.current.add(timerr!, forMode: .common)
    }
    
    override func endRecord(){
        super.endRecord()
        
        timerr?.invalidate()
        timerr = nil
        BFLog(1, message: " 拍摄时长 \(Date().timeIntervalSince(startTime))")
        let su = BFRecordScreenCameraManager.firstOpenCamera
        movieWrite?.finishRecording(completionHandler: {[weak self] in
            guard let wself = self else { return }
            
            if FileManager.default.fileExists(atPath: vpath) {
                if !su {
                    if let finalPath = PQBridgeObject.p_setupFileRename(vpath), finalPath.count > 0 {
                        let dur = AVURLAsset(url: URL(fileURLWithPath: finalPath))
                        wself.videoModel.timelineOut = wself.videoModel.timelineIn + dur.duration.seconds
                        BFLog(1, message: "拍摄文件时长：\(wself.videoModel.timelineIn)~\(wself.videoModel.timelineOut),  \(dur.duration.seconds)")
                        wself.videoModel.locationPath = finalPath
                        wself.currentAssetProgress = CMTime(seconds: wself.videoModel.timelineOut, preferredTimescale: 1000)
                    }else{
                        // 录制失败后恢复原样
                        wself.revertLast()
                        cShowHUB(superView: nil, msg: "录制失败，未能生成文件")
                    }
                    DispatchQueue.main.asyncAfter(deadline: .now() + 0.1) {
                        wself.updateUI(progress:  wself.currentAssetProgress)
                    }
                    wself.recordEndCallBack?(true, wself.videoModel)

                }else{
                    wself.hasInitCallBack?()
                }
            }
            
            // 重置writer，提前为下次录制做准备
            wself.initerlizeWriter()
        })
    }
    
    // 录制失败 ， 不足一秒时恢复上一次状态
    func revertLast(){
        videoModel.timelineOut = videoModel.timelineIn
        currentAssetProgress = CMTime(seconds: videoModel.timelineOut, preferredTimescale: 1000)
        recordItem?.thumbImgs.removeAll(where: { img in
            videoModel.thumImgs?.contains(img) ?? false
        })
        videoModel.thumImgs?.removeAll()
        
        cameraProgressV?.collectionV.reloadData()
    }
    
    override func play() {
        camera?.pauseCapture()
    }
    
    override func pause() {
        camera?.resumeCameraCapture()
    }
    
    override func clean() {
        filter.removeTarget(movieWrite)
        filter.removeTarget(rendView)
        camera?.removeTarget(filter)
        super.clean()
    }
    
    // MARK: - 私有方法
    func updateUI(progress:CMTime){
        DispatchQueue.main.async {[weak self] in
            guard let wself = self else { return }
            
            wself.progreddL?.text = CMTimeGetSeconds(progress).formatDurationToHMS()
            if wself.recordItem?.thumbImgs.count ?? 0 > 0 {
                wself.cameraProgressV?.updateCellWidth(index: wself.recordItem!.thumbImgs.count - 1, progress: progress)
            }
            
        }
    }
    
    func addNewThumb(){
        if let progressThumV = cameraProgressV {
//            progressThumV.collectionV.reloadData()
            progressThumV.addThumbImages()
        }
    }
    
    func beginRecord(startTime:CMTime) {

    }
    
    func initerlizeWriter(){
        if FileManager.default.fileExists(atPath: vpath){
            try? FileManager.default.removeItem(atPath: vpath)
        }
        if movieWrite != nil {
            filter.removeTarget(movieWrite)
            movieWrite = nil
        }
        
        movieWrite = GPUImageMovieWriter(movieURL: URL(fileURLWithPath: vpath), size: videoSize)
        filter.addTarget(movieWrite)

    }
    
}

extension BFRecordScreenCameraManager : GPUImageVideoCameraDelegate {
    func willOutputSampleBuffer(_ sampleBuffer: CMSampleBuffer!) {
        if !recording || BFRecordScreenCameraManager.firstOpenCamera {
            return
        }
        let dur = CMTime(seconds: movieWrite?.duration.seconds ?? 0, preferredTimescale: 1000)
        // 获取缩略图
        let coculationCount = Int(dur.seconds / 5.0)
        if coculationCount >= currVideoThumImagesNum{
            if let img = imageFromBuffer(sampleBuffer){
                BFLog(1, message: "取一张缩略图出来 \(dur.seconds)， \(recordItem?.thumbImgs.count ?? 0)")
                recordItem?.thumbImgs.append(img)
                videoModel.thumImgs?.append(img)
                currVideoThumImagesNum += 1
                addNewThumb()
            }
        }
    }
    
    func imageFromBuffer(_ buff: CMSampleBuffer) -> UIImage? {
        if let buffer = CMSampleBufferGetImageBuffer(buff) {
            let ciImage = CIImage(cvPixelBuffer: buffer)

            return UIImage(ciImage: ciImage.oriented(forExifOrientation: 6),
                           scale: cScreenWidth / 50.0,
                           orientation: .up)

        }

        return nil
    }
    
}
